import Vue from 'vue'
// import 'es6-promise/auto'
import {createApp} from './app'
import ProgressBar from './component/progressbar.vue'

// global progress bar
const bar = Vue.prototype.$bar = new Vue(ProgressBar).$mount();
document.body.appendChild(bar.$el);

// a global mixin that calls `asyncData` when a route component's params change
Vue.mixin({
  beforeRouteUpdate (to, from, next) {
    const {asyncData} = this.$options;
    if (asyncData) {
      asyncData({
        store: this.$store,
        route: to
      }).then(next).catch(next)
    } else {
      next()
    }
  },

  beforeMount () {
    if(this.$store.state.SSR_FETCHED) {
      return
    }

    const {asyncData} = this.$options
    if (asyncData) {
      bar.start();
      asyncData({
        store: this.$store,
        route: this.$route
      }).then(() => {
        bar.finish();
      })
    }
  },

  mounted() {
    this.$store.state.SSR_FETCHED = false
  }
});

const {app, router, store} = createApp();

// prime the store with server-initialized state.
// the state is determined during SSR and inlined in the page markup.
if (window.__INITIAL_STATE__) {
  store.replaceState(window.__INITIAL_STATE__)
}

// wait until router has resolved all async before hooks
// and async components...
router.onReady(() => {
  // router.beforeResolve((to, from, next) => {
  //   const matched = router.getMatchedComponents(to);
  //   const prevMatched = router.getMatchedComponents(from);
  //   let diffed = false;
  //   const activated = matched.filter((c, i) => {
  //     return diffed || (diffed = (prevMatched[i] !== c))
  //   });
  //   const asyncDataHooks = activated.map(c => c.asyncData).filter(_ => _);
  //   if (!asyncDataHooks.length) {
  //     return next()
  //   }
  //
  //   bar.start();
  //
  //   Promise.all(asyncDataHooks.map(hook => hook({ store, route: to })))
  //     .then(() => {
  //       bar.finish();
  //       next()
  //     })
  //     .catch(next)
  // });

  // actually mount to DOM
  app.$mount('#app')
});

// service worker
// if ('https:' === location.protocol && navigator.serviceWorker) {
//   navigator.serviceWorker.register('/service-worker.js')
// }
